from django.utils.functional import cached_property
from django.views.generic import UpdateView
from rules.contrib.views import AutoPermissionRequiredMixin


class HtmxUpdateView(AutoPermissionRequiredMixin, UpdateView):
    fragments = []

    @cached_property
    def is_htmx(self):
        return self.request.headers.get("HX-Request")

    @property
    def permission_type(self):
        if self.request.method == "POST" or getattr(
            self, "_test_write_permission", False
        ):
            return "change"
        return "view"

    def has_change_permission(self):
        self._test_write_permission = True
        permission = self.get_permission_required()[0]
        self._test_write_permission = False
        return self.request.user.has_perm(permission, self.get_permission_object())

    def get_context_data(self, **kwargs):
        result = super().get_context_data(**kwargs)
        result["has_change_permission"] = self.has_change_permission()
        return result

    def _get_fragment(self):
        if self.request.method == "POST":
            fragment = self.request.POST.get("fragment")
        else:
            fragment = self.request.GET.get("fragment")
        if fragment and fragment in self.fragments:
            return fragment

    def get_template_names(self):
        template_names = super().get_template_names()
        if self.is_htmx and (fragment := self._get_fragment()):
            return [f"{template_names[0]}#{fragment}"]
        return template_names

    def get_form_kwargs(self):
        result = super().get_form_kwargs()
        if self.is_htmx:
            data = (
                self.request.POST if self.request.method == "POST" else self.request.GET
            )
            result["single_field"] = data.get("hx-single-field")
        return result

    def form_valid(self, form):
        result = super().form_valid(form)
        if self.is_htmx and self._get_fragment():
            return self.get(self.request, *self.args, **self.kwargs)
        return result
